'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path = require('path');

var _crossSpawn = require('cross-spawn');

var _crossSpawn2 = _interopRequireDefault(_crossSpawn);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _exists = require('./utils/exists');

var _exists2 = _interopRequireDefault(_exists);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Checks path object is string
 * @param  {String} path            Path object, hopefully, a string
 * @param  {String} functionCalling name of function calling
 * @return {Void}
 */
function checkPath(path) {
  if (typeof path !== 'string') {
    var err = new TypeError('Provided path must be a string.');
    throw err;
  }
}

/** Class representing a RAW-format photo and it's information */

var Rawly = function () {
  /**
   * Create an instance of Rawly
   * @param {String} path - Absolute path to file
   */
  function Rawly(path) {
    _classCallCheck(this, Rawly);

    checkPath(path);
    var fileExists = (0, _exists2.default)(path);
    if (fileExists instanceof Error) throw fileExists;

    this.name = Rawly.getFileName(path);
    this.path = Rawly.getDirPath(path);
    this.ext = Rawly.getFileExtension(path);
    this.type = Rawly.getMIMEType(path);
    this.previews = Rawly.getPreviews(path);
    this.previewsExtracted = Rawly.previewsAlreadyExtracted(path);
  }

  /**
   * Get the file name of a certain file
   * @param  {String} path - Absolute path to file
   * @return {String}        String representing file name
   */


  _createClass(Rawly, [{
    key: 'extractPreviews',


    /**
     * Extract previews from RAW-file
     * @param  {(Number|Array<Number>)} [id] - Id of one, or several previews to extract
     * @param  {Boolean}         force=false - Force an overwrite of existing previews
     * @return {Promise}                       Resolves Promise if files are extracted or not
     */
    value: function extractPreviews(id) {
      var _this = this;

      var force = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];

      if (!force && this.previewsExtracted) {
        return _bluebird2.default.resolve(false);
      }

      var args = void 0;

      if (Array.isArray(id)) {
        args = id;
      } else if (id) {
        args = [id];
      } else {
        args = this.previews.map(function (pre) {
          return pre.id;
        });
      }

      return new _bluebird2.default(function (resolve, reject) {
        var previews = args.map(function (arg) {
          return '-ep' + arg;
        });
        var file = (0, _path.join)(_this.path, _this.name + '.' + _this.ext);
        var exiv2 = (0, _crossSpawn2.default)('exiv2', [].concat(_toConsumableArray(previews), [file]));

        exiv2.stdin.setEncoding('utf-8');

        exiv2.stdout.on('data', function () {
          var message = force ? 'yes\n' : 'no\n';
          exiv2.stdin.write(message);
        });

        exiv2.stderr.on('data', function (err) {
          var error = err.toString();
          reject(error);
        });

        exiv2.on('close', function (code) {
          exiv2.stdin.end();
          if (code !== 0) reject(new Error('Exiv2 exited with code ' + code));
          _this.previewsExtracted = true;
          resolve(true);
        });
      });
    }
  }], [{
    key: 'getFileName',
    value: function getFileName(path) {
      checkPath(path);

      var fileName = path.split('/').pop().split('.').slice(0, -1).join('.');

      return fileName;
    }

    /**
     * Get path to firectory where file is located
     * @param  {String} path - Absolute path to file
     * @return {String}        String representing absolute path to directory
     */

  }, {
    key: 'getDirPath',
    value: function getDirPath(path) {
      checkPath(path);

      var dirPath = path.split('/').slice(0, -1).join('/');

      return dirPath;
    }

    /**
     * Get file extension
     * @param  {String} path - Absolute path to file
     * @return {String}        String representing file extension
     */

  }, {
    key: 'getFileExtension',
    value: function getFileExtension(path) {
      checkPath(path);

      var extension = path.split('/').pop().split('.').pop();

      return extension;
    }

    /**
     * Get MIME type of file provided
     * @param  {String} path - Absolute path to file
     * @return {String}        String representing MIME type (eg. "image/x-canon-cr2")
     */

  }, {
    key: 'getMIMEType',
    value: function getMIMEType(path) {
      checkPath(path);

      var result = _crossSpawn2.default.sync('exiv2', ['pr', path]).output[1].toString();
      var mimetype = result.match(/MIME type\s*:\s(.*)/);
      return mimetype[1];
    }

    /**
     * Get the files available previews
     * @param  {String}        path - Absolute path to file
     * @return {Array<Object>}        An array of objects
     */

  }, {
    key: 'getPreviews',
    value: function getPreviews(path) {
      checkPath(path);

      var result = _crossSpawn2.default.sync('exiv2', ['-pp', path]).output[1].toString().trim().split('\n');

      try {
        var previews = result.map(function (string) {
          var _string$match = string.match(/\w*\s(\d*):\s(\w*\/\w*),\s(\d*x\d*)\s\w*,\s(\d*)\s\w*/i);

          var _string$match2 = _slicedToArray(_string$match, 5);

          var id = _string$match2[1];
          var type = _string$match2[2];
          var dimensions = _string$match2[3];
          var size = _string$match2[4];


          return { id: Number(id), type: type, dimensions: dimensions, size: Number(size) };
        });

        return previews;
      } catch (err) {
        return undefined;
      }
    }

    /**
     * Check if a preview is aready created
     * @param {Sting}    path - Absolute path to file
     * @return {Boolean}        If a preview already exists of not
     */

  }, {
    key: 'previewsAlreadyExtracted',
    value: function previewsAlreadyExtracted(path) {
      checkPath(path);
      var file = {
        name: Rawly.getFileName(path),
        ext: Rawly.getFileExtension(path),
        dir: Rawly.getDirPath(path)
      };

      var globExp = file.dir + '/' + file.name + '?(-preview*).!(' + file.ext + ')';
      var result = _glob2.default.sync(globExp);

      if (result.length > 0) return true;

      return false;
    }
  }]);

  return Rawly;
}();

exports.default = Rawly;